@@ -1,5 +1,5 @@
 import logging
-from typing import Literal, Union, List
+from typing import Literal, Optional, Union, List
 import requests
 import tempfile
 import pandas as pd
@@ -53,7 +53,7 @@ class BreatheDesignModel:
         device_auth.get_token()
 
     def _make_api_call(
-        self, verb: Literal["POST", "GET"], endpoint: str, payload: dict | None
+        self, verb: Literal["POST", "GET"], endpoint: str, payload: Optional[dict]
     ) -> dict:
         token = device_auth.get_token()
         headers = {
@@ -79,11 +79,15 @@ class BreatheDesignModel:
             if response.status_code == 200:
                 return response.json()
             else:
-                raise BreatheException(f"Error {response.status_code}: {response.text}")
+                trace_id = response.headers.get("x-trace-id")
+                span_id = response.headers.get("x-span-id")
+                raise BreatheException(
+                    f"Error {response.status_code}, trace_id: {trace_id}, span_id: {span_id}: {response.text}"
+                )
         except Exception as e:
             raise BreatheException(f"Error {e}")
 
-    def _make_model_api_call(self, endpoint: str, payload: dict | None) -> dict:
+    def _make_model_api_call(self, endpoint: str, payload: Optional[dict]) -> dict:
         full_endpoint = f"/model-services/pbm/{endpoint}"
         return self._make_api_call("POST", full_endpoint, payload)
 
@@ -108,8 +112,7 @@ class BreatheDesignModel:
         _logger.info("Getting batteries...")
         batteries = self._make_api_call("GET", "/cells/", None)
 
-        # unpack the ID and model name
-        return [b["pbm_model_name"] for b in batteries]
+        return [b["name"] for b in batteries]
 
     def get_battery_format(self, base_battery: str) -> dict:
         """
