@@ -1,10 +1,18 @@
 import random
-from typing import Any
+from typing import Any, Optional, Union
 import plotly.graph_objects as go
 from plotly.subplots import make_subplots
 import numpy as np
 import pandas as pd
 
+import os
+import plotly.io as pio
+
+pio.renderers.default = os.getenv(
+    "BREATHE_PLOTLY_RENDERER", "plotly_mimetype+notebook_connected"
+)
+
+
 # =====================================================================
 # Monte Carlo Generator
 # =====================================================================
@@ -13,10 +21,10 @@ import pandas as pd
 def make_manufacturing_variations(
     base_design: dict[str, Any],
     n_samples: int = 10,
-    bound_range: float | list[float] = 0.05,
-    sigma_fraction: float | list[float] | None = None,
-    seed: int | None = None,
-    param_pairs: list[tuple[str, str]] | None = None,
+    bound_range: Union[float, list[float]] = 0.05,
+    sigma_fraction: Optional[Union[float, list[float]]] = None,
+    seed: Optional[int] = None,
+    param_pairs: Optional[list[tuple[str, str]]] = None,
     round_ndigits: int = 5,
     eps: float = 1e-12,
     design_name_format: str = "MC Variation sample #%i",
@@ -49,7 +57,6 @@ def make_manufacturing_variations(
     if seed is not None:
         random.seed(seed)
 
-    # Infer pairs if not provided
     if param_pairs is None:
         exclude = {"anode", "cathode"}
         anode_keys = [
@@ -72,7 +79,6 @@ def make_manufacturing_variations(
     if not param_pairs:
         raise ValueError("No numeric anode/cathode parameter pairs found.")
 
-    # turn bound_range into list
     if isinstance(bound_range, float):
         bound_range = [bound_range] * len(param_pairs)
     elif len(bound_range) != len(param_pairs):
@@ -80,7 +86,6 @@ def make_manufacturing_variations(
             "bound_range must be a float or list of floats with the same length as param_pairs"
         )
 
-    # make default sigma fraction if not provided, otherwise ensure it is a list of the correct length
     if sigma_fraction is None:
         sigma_fraction = [br / 3.0 for br in bound_range]
     elif isinstance(sigma_fraction, float):
@@ -113,15 +118,12 @@ def make_manufacturing_variations(
             mean_an, sigma_an, lo_an, hi_an = stats[an_key]
             mean_ca, sigma_ca, lo_ca, hi_ca = stats[ca_key]
 
-            # sample
             z_an = random.gauss(0.0, 1.0)
             z_ca = random.gauss(0.0, 1.0)
 
-            # scale samples, and clamp within bounds
             an_val = min(max(mean_an + z_an * sigma_an, lo_an), hi_an)
             ca_val = min(max(mean_ca + z_ca * sigma_ca, lo_ca), hi_ca)
 
-            # round
             an_rounded = round(
                 an_val + (0 if abs(an_val - mean_an) > eps else 0.0), round_ndigits
             )
@@ -157,9 +159,9 @@ def plot_scatter_matrix(
     base_design: dict[str, Any],
     changes: list[dict[str, Any]],
     param_keys: list[str],
-    title: str = "Manufacturing variations",  # updated default title
+    title: str = "Manufacturing variations",
     marker_size: int = 6,
-    tickangle: int | None = -45,  # tilt labels if needed
+    tickangle: Optional[int] = -45,
 ) -> go.Figure:
     """Create a Plotly scatter-plot matrix (SPLOM) for selected parameters.
 
@@ -173,14 +175,12 @@ def plot_scatter_matrix(
     Returns:
         (plotly.Figure): A Plotly figure object.
     """
-    # Reconstruct full samples
     samples = []
     for ch in changes:
         d = dict(base_design)
         d.update(ch)
         samples.append(d)
 
-    # Extract data
     data = {k: [s.get(k, None) for s in samples] for k in param_keys}
 
     fig = go.Figure(
@@ -197,22 +197,22 @@ def plot_scatter_matrix(
         )
     )
 
-    # Layout styling
     fig.update_layout(
-        title=dict(text=title, font=dict(size=14)),  # smaller title font
+        title=dict(text=title, font=dict(size=14)),
         width=800,
         height=600,
         paper_bgcolor="rgba(240,240,240,1)",
         plot_bgcolor="rgba(240,240,240,1)",
-        font=dict(size=8),  # << shrink dimension labels more
+        font=dict(size=8),
     )
 
-    # Reduce tick labels further & tilt
     fig.update_xaxes(tickfont=dict(size=8), showgrid=True)
     fig.update_yaxes(tickfont=dict(size=8), showgrid=True)
     if tickangle is not None:
         fig.update_xaxes(tickangle=tickangle)
 
+    fig.show()
+
     return fig
 
 
@@ -237,7 +237,6 @@ def plot_variation_histograms(
     Returns:
         (plotly.Figure): A Plotly figure object.
     """
-    # Reconstruct full samples
     samples = []
     for ch in changes:
         d = dict(base_design)
@@ -273,6 +272,8 @@ def plot_variation_histograms(
         plot_bgcolor="rgba(240,240,240,1)",
         bargap=0.05,
     )
+    fig.show()
+
     return fig
 
 
@@ -292,10 +293,8 @@ def plot_capacity_histogram(df, title="Distribution of Capacity [Ah]", bins=20):
     if "Capacity [Ah]" not in df.index:
         raise KeyError("'Capacity [Ah]' row not found in DataFrame index.")
 
-    # Extract capacity row (all designs, including Baseline + MC Variations)
     values = df.loc["Capacity [Ah]"].values
 
-    # Create histogram
     fig = go.Figure(
         data=go.Histogram(
             x=values,
@@ -305,7 +304,6 @@ def plot_capacity_histogram(df, title="Distribution of Capacity [Ah]", bins=20):
         )
     )
 
-    # Apply styling
     fig.update_layout(
         title=title,
         xaxis_title="Capacity [Ah]",
@@ -317,6 +315,8 @@ def plot_capacity_histogram(df, title="Distribution of Capacity [Ah]", bins=20):
         plot_bgcolor="rgba(240,240,240,1)",
     )
 
+    fig.show()
+
     return fig
 
 
@@ -339,7 +339,6 @@ def plot_dcir_histogram(
     if "Resistance (Ω)" not in df.columns:
         raise KeyError("'Resistance (Ω)' column not found in DataFrame.")
 
-    # Optionally exclude Baseline
     if not include_baseline and "Design" in df.columns:
         values = df.loc[df["Design"] != "Baseline", "Resistance (Ω)"].values
     else:
@@ -349,14 +348,11 @@ def plot_dcir_histogram(
         data=go.Histogram(
             x=values,
             nbinsx=bins,
-            marker=dict(
-                color=hex_colors[1]
-            ),  # use second Breathe color for distinction
+            marker=dict(color=hex_colors[1]),
             opacity=0.75,
         )
     )
 
-    # Apply styling
     fig.update_layout(
         title=title,
         xaxis_title="Resistance (Ω)",
@@ -368,6 +364,8 @@ def plot_dcir_histogram(
         plot_bgcolor="rgba(240,240,240,1)",
     )
 
+    fig.show()
+
     return fig
 
 
@@ -411,7 +409,6 @@ def plot_capacity_resistance_heatmap(
     if capacity_row_label not in kpi_df.index:
         raise KeyError(f"'{capacity_row_label}' not found in kpi_df index.")
 
-    # capacity row: index=Design, values=Capacity
     cap = (
         kpi_df.loc[capacity_row_label]
         .to_frame(name=capacity_row_label)
@@ -423,7 +420,6 @@ def plot_capacity_resistance_heatmap(
     if not include_baseline:
         df = df[df[design_col] != "Baseline"]
 
-    # reconstruct design parameters (baseline + changes)
     full_designs = []
     for i, ch in enumerate(changes, start=1):
         d = dict(base_design)
@@ -437,7 +433,6 @@ def plot_capacity_resistance_heatmap(
 
     full_designs = {d["Design"]: d for d in full_designs}
 
-    # Merge parameters into hover text
     hover_texts = []
     for _, row in df.iterrows():
         design_name = row[design_col]
@@ -450,7 +445,6 @@ def plot_capacity_resistance_heatmap(
     x = df[capacity_row_label].astype(float).to_numpy()
     y = df[resistance_col].astype(float).to_numpy()
 
-    # density coloring
     x_edges = np.linspace(x.min(), x.max(), bins[0] + 1)
     y_edges = np.linspace(y.min(), y.max(), bins[1] + 1)
     counts, _, _ = np.histogram2d(x, y, bins=[x_edges, y_edges])
@@ -460,7 +454,6 @@ def plot_capacity_resistance_heatmap(
     if density.max() > 0:
         density = density / density.max()
 
-    # plot
     fig = go.Figure(
         data=go.Scatter(
             x=x,
@@ -487,4 +480,6 @@ def plot_capacity_resistance_heatmap(
         plot_bgcolor="rgba(240,240,240,1)",
     )
 
+    fig.show()
+
     return fig
