@@ -5,6 +5,7 @@ This module provides convenient interfaces for working with simulation results,
 including easy access to KPIs, dynamic data, and integrated plotting capabilities.
 """
 
+from typing import Optional
 import pandas as pd
 import plotly.graph_objects as go
 from plotly.subplots import make_subplots
@@ -84,7 +85,7 @@ class SingleSimulationResults:
         else:
             raise ValueError("Capacity KPI not found in results")
 
-    def get_kpis(self, design_name: str | None = None) -> pd.DataFrame:
+    def get_kpis(self, design_name: Optional[str] = None) -> pd.DataFrame:
         """Get KPIs for specific design(s)."""
         if design_name is None:
             return self.kpis
@@ -94,7 +95,7 @@ class SingleSimulationResults:
             )
         return self.kpis[design_name]
 
-    def get_normalized_kpis(self, design_name: str = None) -> pd.DataFrame:
+    def get_normalized_kpis(self, design_name: Optional[str] = None) -> pd.DataFrame:
         """
         Get normalized KPIs relative to the Baseline design.
 
@@ -119,7 +120,7 @@ class SingleSimulationResults:
 
         return normalize_results(kpis)
 
-    def get_dynamic_data(self, design_name: str | None = None) -> dict:
+    def get_dynamic_data(self, design_name: Optional[str] = None) -> dict:
         """Get dynamic data for specific design(s)."""
         if design_name is None:
             return self.dynamic_data
@@ -130,7 +131,7 @@ class SingleSimulationResults:
         )
 
     def compare_designs(
-        self, plot_type: str = "relative", designs: list[str] | None = None
+        self, plot_type: str = "relative", designs: Optional[list[str]] = None
     ) -> go.Figure:
         """Compare designs using KPI plots."""
         kpis = self.kpis
@@ -146,12 +147,12 @@ class SingleSimulationResults:
         else:
             return plot_kpis(kpis, plot_type)
 
-    def plot_voltage_response(self, design_name: str | None = None) -> go.Figure:
+    def plot_voltage_response(self, design_name: Optional[str] = None) -> go.Figure:
         """Plot voltage response for specified design."""
         return self.plot_dynamic_response("voltModel", design_name)
 
     def plot_dynamic_response(
-        self, variable_name: str = "voltModel", design_name: str | None = None
+        self, variable_name: str = "voltModel", design_name: Optional[str] = None
     ) -> go.Figure:
         """Plot dynamic response for specified variable and design."""
         dynamic_data = self.get_dynamic_data(design_name)
@@ -160,7 +161,7 @@ class SingleSimulationResults:
         return plot_dynamic_response(dynamic_data, variable_name)
 
     def dynamic_kpis(
-        self, dynamic_kpi_type: str, design_name: str | None = None
+        self, dynamic_kpi_type: str, design_name: Optional[str] = None
     ) -> pd.DataFrame:
         """
         Extract dynamic KPIs for specified type and design.
@@ -208,7 +209,7 @@ class SingleSimulationResults:
         sensitivities = self._result["Sensitivities"]
         return plot_sensitivities_plot(kpi_names, sensitivities)
 
-    def plot_radar(self, designs: list[str] = None) -> go.Figure:
+    def plot_radar(self, designs: Optional[list[str]] = None) -> go.Figure:
         """
         Create a radar plot comparing designs across KPIs.
 
@@ -366,12 +367,12 @@ class BatchSimulationResults:
         """Get dynamic data for all simulations."""
         return [sim.dynamic_data for sim in self._simulation_results]
 
-    def get_kpis(self, designs: list[str] | None = None) -> list[pd.DataFrame]:
+    def get_kpis(self, designs: Optional[list[str]] = None) -> list[pd.DataFrame]:
         """Get KPIs for all the first simulation."""
         return self.first.get_kpis(designs)
 
     def get_normalized_kpis(
-        self, designs: list[str] | None = None
+        self, designs: Optional[list[str]] = None
     ) -> list[pd.DataFrame]:
         """
         Get normalized KPIs for all simulations relative to their Baseline designs.
@@ -387,12 +388,12 @@ class BatchSimulationResults:
         """
         return self.first.get_normalized_kpis(designs)
 
-    def get_dynamic_data(self, design_name: str | None = None) -> list[dict]:
+    def get_dynamic_data(self, design_name: Optional[str] = None) -> list[dict]:
         """Get dynamic data for all simulations."""
         return [sim.get_dynamic_data(design_name) for sim in self._simulation_results]
 
     def compare_designs(
-        self, plot_type: str = "relative", designs: list[str] | None = None
+        self, plot_type: str = "relative", designs: Optional[list[str]] = None
     ) -> go.Figure:
         """Compare designs across all simulations using subplots."""
         if plot_type == "radar":
@@ -430,7 +431,7 @@ class BatchSimulationResults:
         return fig
 
     def plot_dynamic_response(
-        self, variable_name: str = "voltModel", design_name: str | None = None
+        self, variable_name: str = "voltModel", design_name: Optional[str] = None
     ) -> go.Figure:
         """Plot dynamic response across all simulations using subplots."""
         # Check if dynamic data is available for any simulation
@@ -453,12 +454,12 @@ class BatchSimulationResults:
         fig.update_layout(height=300 * rows)
         return fig
 
-    def plot_voltage_response(self, design_name: str | None = None) -> go.Figure:
+    def plot_voltage_response(self, design_name: Optional[str] = None) -> go.Figure:
         """Plot voltage response across all simulations using subplots."""
         return self.plot_dynamic_response("voltModel", design_name)
 
     def dynamic_kpis(
-        self, dynamic_kpi_type: str, design_name: str | None = None
+        self, dynamic_kpi_type: str, design_name: Optional[str] = None
     ) -> list[pd.DataFrame]:
         """
         Extract dynamic KPIs for all simulations.
@@ -485,7 +486,7 @@ class BatchSimulationResults:
         """Plot sensitivity heatmap for the first simulation in the batch."""
         return self.first.plot_sensitivities()
 
-    def plot_radar(self, designs: list[str] | None = None) -> go.Figure:
+    def plot_radar(self, designs: Optional[list[str]] = None) -> go.Figure:
         """
         Create a radar plot comparing designs across KPIs for the first simulation.
 
